package jrummikub.view.impl;

import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;

import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.border.EmptyBorder;

import jrummikub.util.Event;
import jrummikub.util.IEvent;

/**
 * A panel that is displayed when a player has won
 */
@SuppressWarnings("serial")
class WinPanel extends JPanel {
	private final static int PANEL_INSET = 15;
	private final static int PANEL_SEPARATOR = 10;
	@SuppressWarnings("unused")
	private final static float PANEL_FIRST_LINE_HEIGHT = 0.375f;
	private final static int PANEL_MAX_WIDTH = 180;
	private final static float MAX_BUTTON_FONT_SIZE = 12;

	private JButton newRoundButton;
	private JButton newGameButton;
	private JButton endProgramButton;

	private Event endProgramEvent = new Event();
	private Event newGameEvent = new Event();
	private Event newRoundEvent = new Event();

	/**
	 * Creates a new WinPanel
	 */
	WinPanel() {
		setLayout(null);
		setBorder(new EmptyBorder(PANEL_INSET, PANEL_INSET, PANEL_INSET,
				PANEL_INSET));

		newRoundButton = new JButton("Neue Runde");
		newRoundButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				newRoundEvent.emit();
			}
		});
		add(newRoundButton);

		newGameButton = new JButton("Neues Spiel");
		newGameButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				newGameEvent.emit();
			}
		});
		add(newGameButton);

		endProgramButton = new JButton("Programm verlassen");
		endProgramButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				endProgramEvent.emit();
			}
		});
		add(endProgramButton);

		addComponentListener(new ComponentAdapter() {
			@Override
			public void componentResized(ComponentEvent e) {
				rescale();
			}
		});
	}

	/**
	 * The new round event is emitted when the player wants to start a new round
	 * 
	 * @return the event
	 */
	IEvent getNewRoundEvent() {
		return newRoundEvent;
	}

	IEvent getNewGameEvent() {
		return newGameEvent;
	}

	/**
	 * The end program is emitted when the player wants to quit the program
	 * 
	 * @return the event
	 */
	IEvent getEndProgramEvent() {
		return endProgramEvent;
	}

	private void rescale() {
		Insets insets = getInsets();
		int x = insets.left, y = insets.top, width = getWidth() - insets.left
				- insets.right, height = getHeight() - insets.top - insets.bottom;

		if (width > PANEL_MAX_WIDTH) {
			x += (width - PANEL_MAX_WIDTH) / 4;
			width = width / 2 + PANEL_MAX_WIDTH / 2;
		}

		int buttonWidth = (width - 2 * PANEL_SEPARATOR) / 3;
		int buttonHeight = height;
		float fontSize = (float) Math.sqrt(buttonWidth * buttonHeight) / 5;
		if (fontSize > MAX_BUTTON_FONT_SIZE)
			fontSize = MAX_BUTTON_FONT_SIZE;

		newRoundButton.setBounds(x, y, buttonWidth, buttonHeight);
		newRoundButton.setFont(newRoundButton.getFont().deriveFont(fontSize));

		newGameButton.setBounds(x + buttonWidth + PANEL_SEPARATOR, y, buttonWidth,
				buttonHeight);
		newGameButton.setFont(newGameButton.getFont().deriveFont(fontSize));

		endProgramButton.setBounds(x + 2 * (buttonWidth + PANEL_SEPARATOR), y,
				buttonWidth, buttonHeight);
		endProgramButton.setFont(endProgramButton.getFont().deriveFont(fontSize));

	}
}
